% Power curve of the point-optimal test:
% martingale time varition under the alternative hypothesis.
% 
% y  = mu  + u
% mu = e*mu_bar + Be*mu_tilde
%
% \sqrt{T}mu_bar ~ N(0,c_bar*I_k)
% T*mu_tilde     ~ N(O, Be'*F*F'*Be x c_tilde*I_k)
%
% c_bar   = (1-omega)*c
% c_tilde = omega*c
% omega   = weight on the stability part of the test 
% beta    = structural parameter of interest
%
% Under H0: mu_bar=0 and mu_tilde=0 (no break)
% Need parallel computing toolbox

tic

clear

%% Parameters and Instruments

T     = 2000;         % # of observations
N     = 20000;        % # of Monte Carlo simulations
m     = 1;            % # of endogenous variables
k     = 1;            % # of excluded instruments
omega = 0;            % weight

root_c      = [0.00001 0.25 0.5 0.75 1:1:8 9:2:25]';
sc          = length(root_c);
vC          = root_c.^2;   
vC_bar      = vC*(1-omega);
rootvC_bar  = vC_bar.^.5;
vC_tilde    = vC*omega;
rootvC_tilde= vC_tilde.^.5;
weight_full = vC_bar./(1+vC_bar);   %see equation (9)

%-- DGP under H0
stream_U    = RandStream('mt19937ar','Seed',0); % fix rand generating number
mU          = [zeros(1,k,N); randn(stream_U,T-1,k,N)]; % data under H0
%-- DGP of mu_tilde under H1: variance ajusted inside the loop
stream_eta  = RandStream('mt19937ar','Seed',2);
mEta        = [zeros(1,k,N); randn(stream_eta,T-1,k,N)];
%-- DGP of mu_bar under H1: variance ajusted inside the loop
stream_epsilon = RandStream('mt19937ar','Seed',3);
mEpsilon    = randn(stream_epsilon,N,k);

alpha  = 0.05;               % significance level
prob   = 0.15;               % trimming parameter
Ttau_l = floor(prob*T);
Ttau_u = floor((1-prob)*T);

%% Storage Matrices -   
GMM_S           = zeros(N,sc);        
ave_S           = zeros(N,sc);       
exp_S           = zeros(N,sc);
qLL_S           = zeros(N,sc);
qLL_S_opt       = zeros(N,sc);
qLL_S_opt0      = zeros(N,sc); % Distribution under the H0
ave_Stilde      = zeros(N,sc);       
exp_Stilde      = zeros(N,sc);
qLL_Stilde      = zeros(N,sc);
qLL_Stilde_opt  = zeros(N,sc);
qLL_Stilde_opt0 = zeros(N,sc); % Distribution under the H0

%% declare variables used inside the loop
vOnes = ones(T,1) ; Y_DGP = zeros(T,k); M_vr_opt = zeros(T);
mu_bar= zeros(m,k); DY_DGP=zeros(T,k) ; 
Eta=zeros(T,k)    ; U=zeros(T,k)      ; bm=zeros(T,k); bb=zeros(T,k); 

S_T  = zeros(1+Ttau_u-Ttau_l,1);
BS_T = zeros(1+Ttau_u-Ttau_l,1);

%-- Variables for computing the S-tilde tests
s             = (1:1:T)'/T;           % column vector [1 1/T 2/T ... 1]
s_tau         = s(Ttau_l:Ttau_u,:);
s_1mtau       = 1 - s_tau;
s_tau_x_1mtau = s_tau.*s_1mtau;
sqrt_T        = sqrt(T);

%-- Variables for the dgp
iota = ones(T,1) ; M_i = eye(T)-iota*((iota'*iota)\iota'); 
F = tril(ones(T)); M_iF = M_i*F;

%-- Variable for all qll-tests
I_T = eye(T); ones_Tm1= ones(T-1,1); Toprows=[1 zeros(1,T-1)];

%-- Variables  of the qLL-S test
c_10    = 10; 
wf      = c_10/(1+c_10); 
rtilde  = 1 - c_10/T;
seqT    = (1:1:T)';
vr      = rtilde.^seqT;
vr_1    = [1; vr(1:T-1)];
M_vr    = I_T - vr*((vr'*vr)\vr');
seqT    = (1:1:T)';
matlabpool local 4 %parallel computing function
for i=1:sc
    beta         = root_c(i); 
    root_c_bar   = rootvC_bar(i); 
    root_c_tilde = rootvC_tilde(i);
    wf_opt       = weight_full(i);
    %-- Variables of the qLL-S optimal test
    rtilde_opt   = 1 - root_c_tilde/T;
    vr_opt       = rtilde_opt.^seqT;
    M_vr_opt     = I_T - vr_opt*((vr_opt'*vr_opt)\vr_opt');
    vr_opt1      = [1; vr_opt(1:T-1)];
    %for n=1:N
    parfor n=1:N      %parallel computing function 
        %-- Generating values for the model
        Eta        = mEta(:,1:k,n);
        mu_bar     = root_c_bar*mEpsilon(n,:)/sqrt_T;
        
        %% Data under H0
        %-- qLL-S point optimal test - critical values
        U     = mU(:,1:k,n);
        mF_T  = sum(U)/T;
        S_0   = mF_T*(mF_T')*T;
                
        DY_DGP  = U-vOnes*mF_T;
        vMv     = sum(sum(DY_DGP.^2),2);
        ini     = [U(1,:);diff(U)];
        mwopt   = filter(vr_opt1,1,ini);
        e       = M_vr_opt*mwopt;      % least squares residuals
        e2      = sum(sum(e.^2,1),2);
        
        qLL_Stilde_opt0(n,i)= -e2*rtilde_opt + vMv; 
        qLL_S_opt0(n,i)     = wf_opt*S_0+ qLL_Stilde_opt0(n,i);
        
        %% Data under H1
        Y_DGP       = iota*mu_bar + root_c_tilde*M_iF*Eta/T + U;
        mF_T        = sum(Y_DGP)/T;
        GMM_S(n,i) = T*mF_T*(mF_T');
        
        %-- qLL-S test
        DY_DGP= Y_DGP-vOnes*mF_T;
        vMv   = sum(sum(DY_DGP.^2),2);
        ini   = [Y_DGP(1,:);diff(Y_DGP)];
        mw    = filter(vr_1,1,ini);
        e     = M_vr*mw;
        e2    = sum(sum(e.^2,1),2);
        
        qLL_Stilde(n,i)= -e2*rtilde + vMv;
        qLL_S(n,i)     =  wf*GMM_S(n,i) + qLL_Stilde(n,i);
                
        %-- qLL-S point optimal test
        mw_opt = filter(vr_opt1,1,ini);
        e      = M_vr_opt*mw_opt;
        e2     = sum(sum(e.^2,1),2);
        
        qLL_Stilde_opt(n,i)= -e2*rtilde_opt + vMv; 
        qLL_S_opt(n,i)= wf_opt*GMM_S(n,i) + qLL_Stilde_opt(n,i);
        
        %%
        bm   = cumsum(Y_DGP)/sqrt_T;
        bb   = bm-s*bm(T,:);     % B(s) - s*B(1) brownian bridge
    
        A    = bb(Ttau_l:Ttau_u,:);
        BS_T = sum(A.*A,2)./s_tau_x_1mtau;
        chi  = bm(T,:)*bm(T,:)'; % random number from chi-square dist.
                
        ave_Stilde(n,i) = mean(BS_T);
        exp_Stilde(n,i) = 2*log(mean(exp(.5*BS_T)));
        
        ave_S(n,i) = ave_Stilde(n,i)+chi;
        exp_S(n,i) = exp_Stilde(n,i)+chi;
                     
        
    end
end
%% Computing the critical values
cv_S        = quantile(GMM_S(:,1),1-alpha);
cv_ave_S    = quantile(ave_S(:,1),1-alpha);
cv_exp_S    = quantile(exp_S(:,1),1-alpha);
cv_qLL_S    = quantile(qLL_S(:,1),1-alpha);
cv_qLL_S_opt= quantile(qLL_S_opt0,1-alpha);

cv_ave_Stilde     = quantile(ave_Stilde(:,1),1-alpha);
cv_exp_Stilde     = quantile(exp_Stilde(:,1),1-alpha); 
cv_qLL_Stilde     = quantile(qLL_Stilde(:,1),1-alpha);  
cv_qLL_Stilde_opt = quantile(qLL_Stilde_opt0,1-alpha);


% Power Curves
pw_GMM_S = mean(GMM_S>cv_S); 
pw_ave_S = mean(ave_S>cv_ave_S);
pw_exp_S = mean(exp_S>cv_exp_S);
pw_qLL_S = mean(qLL_S>cv_qLL_S);
pw_qLL_S_opt = mean(qLL_S_opt>ones(size(qLL_S_opt,1),1)*cv_qLL_S_opt);

pw_ave_Stilde = mean(ave_Stilde>cv_ave_Stilde);
pw_exp_Stilde = mean(exp_Stilde>cv_exp_Stilde);
pw_qLL_Stilde = mean(qLL_Stilde>cv_qLL_Stilde);
pw_qLL_Stilde_opt = mean(qLL_Stilde_opt>...
        ones(size(qLL_Stilde_opt,1),1)*cv_qLL_Stilde_opt);


%% - Plot the Power Curves
bbb = min(root_c):.01:max(root_c);
ff  = 0.05;

save  PO_power_mart_k1_omega00  root_c pw* bbb ff 

matlabpool close % parallel computing function 

bbb = min(root_c):.2:max(root_c);
ff  = 0.05;

figure (3)
plot1=plot(root_c,pw_GMM_S(:),...
     root_c,pw_ave_S(:),...
     root_c,pw_exp_S(:),...
     root_c,pw_qLL_S(:),...
     root_c,pw_qLL_S_opt(:),...
     'LineWidth',3.0,'MarkerSize',5);
set(plot1(1),'LineWidth',2.5,'Color',[0 0 1]);
set(plot1(2),'LineStyle',':','LineWidth',4,'Color',[0 0 0]);
set(plot1(3),'LineWidth',2.5,'LineStyle','--','Color',[1 0 0]);
set(plot1(4),'LineStyle','-.','LineWidth',2.5,'Color',[0.749 0 0.749]);
set(plot1(5),'LineStyle','-','LineWidth',1,'Color',[0 0 0]);
set(gca,'XTick',[0 5 10 15 20 25],'YTick',[0.2 .40 .60 .80 1.0],'YMinorTick','on','FontSize',14)
axis([min(root_c) max(root_c) 0 1]);
xlabel('$\sqrt{c}$','Interpreter','latex','FontSize',17);
ylabel('rejection probability','FontSize',16);
% legend('S','ave-S','exp-S','qLL-S','opt-S','Location',[0.58 0.20 .42 .30]);
% legend ('boxoff')
saveas(gcf,'mart_k1_omega00', 'pdf')


toc

