% Power curve of the point-optimal test:
% Single break: unknown  date
% 
% sqrt(T)*m(s)    = 1(s<tau)*m1+1(s>=tau)*m2
% sqrt(T)*m_bar   = (tau*m1 + (1-tau)*m2)
% sqrt(T)*delta   = tau*(1-tau)*(m1-m2)
% sqrt(T)*m_tilde = sqrt(T)*[m(s) - m_bar]
%                 = (1(s<tau)-tau)*(m1-m2)
% m1 = mean of the process before the break
% m2 = mean of the process after the break
% T is the number of observations (T=2000)
% sqrt(T)*m_bar   ~ N(0,c_bar*I_k)
% tau             ~ unif(0.15,0.85)
% sqrt(T)*delta   ~ N(0,c_tilde*tau*(1-tau)*I_k)
% c_bar   = (1-omega)*c
% c_tilde = omega*c
% omega   = weight on the stability part of the test 
% beta    = structural parameter of interest

% X       = 1(s<tau)*m1+1(s>=tau)*m2 + N(0,I_T); standardized process
%
% m1 = (1/tau) (tau*m_bar + delta)
% m2 = (1/(1-tau))*( (1-tau)*m_bar - delta )
% Note that under H0: m1=m2=0 (no break)
% Need parallel computing toolbosx

tic

clear

%% Parameters and Instruments

T     = 2000;         % # of observations
N     = 20000;        % # of Monte Carlo simulations
m     = 1;            % # of endogenous variables
k     = 1;            % # of excluded instruments
omega = 0.00;         % weight

root_c      = [0.00001 0.25 0.5 0.75 1:1:8 9:2:25]';
sc          = length(root_c);
vC          = root_c.^2;   
vC_bar      = vC*(1-omega);
vC_tilde    = vC*omega;
weight_full = vC_bar./(1+vC_bar);   %see equation (9)
weight_stab = vC_tilde./(1+vC_tilde);

%-- DGP under the null hypothesis
stream_X = RandStream('mt19937ar','Seed',0); % fix rand generating number
mX       = [zeros(1,k,N); randn(stream_X,T-1,k,N)]; % data under H0

%--- m_bar: variance is ajusted inside the loop
stream_mbar = RandStream('mt19937ar','Seed',2);
mM_br       = randn(stream_mbar,N,k);
%--- delta: variance is ajusted insided the loop
stream_delta= RandStream('mt19937ar','Seed',3);
mDelta      = randn(stream_delta,N,k);

alpha  = 0.05;               % significance level
prob   = 0.15;               % trimming parameter
Ttau_l = floor(prob*T);
Ttau_u = floor((1-prob)*T);
%--- Tau ~ uniform  distribution;
stream_tau = RandStream('mt19937ar','Seed',1);
vTau       = unifinv(rand(stream_tau,N,1),prob,1-prob);
vT1        = floor(T*vTau);

%% Storage Matrices - Reject H0 == 1, Not Reject H0 == 0  
GMM_S      = zeros(N,sc);          
AR_sup      = zeros(N,sc);       
ave_S      = zeros(N,sc);       
exp_S      = zeros(N,sc);
qLL_S      = zeros(N,sc);
exp_S_opt  = zeros(N,sc);
exp_S_opt0 = zeros(N,sc);  % Distribution under the H0
ave_Stilde = zeros(N,sc);       
exp_Stilde = zeros(N,sc);
qLL_Stilde = zeros(N,sc);
exp_Stilde_opt  = zeros(N,sc);
exp_Stilde_opt0 = zeros(N,sc); % Distribution under the H0

       
%% declare variables used inside the loop
vOnes = ones(T,1) ; X     = zeros(T,k); X_DGP = zeros(T,k);
m_bar = zeros(m,k); delta = zeros(m,k);
bm=zeros(T,k); bb=zeros(T,k);
m1    = zeros(m,k);       % mean before the break
m2    = zeros(m,k);       % mean after the break

c_10    = 10;
aux1 = c_10/(1+c_10);
S_T    = zeros(1+Ttau_u-Ttau_l,1);
BS_T   = zeros(1+Ttau_u-Ttau_l,1);

s             = (1:1:T)'/T;           % column vector [1 1/T 2/T ... 1]
s_tau         = s(Ttau_l:Ttau_u,:);
s_1mtau       = 1 - s_tau;
s_tau_x_1mtau = s_tau.*s_1mtau;
sqrt_T        = sqrt(T);

%-- Variables  of the qLL-S test
rtilde = 1 - c_10/T;
vr     = rtilde.^(1:1:T)';
vr_1   = [1; vr(1:T-1)];
M_vr   = eye(T) - vr*((vr'*vr)\vr');

matlabpool local 4 %parallel computing function
for i=1:sc
    beta = root_c(i);
    root_c_bar   = sqrt(vC_bar(i)); 
    root_c_tilde = sqrt(vC_tilde(i));
    
    term1   = weight_stab(i);
    term2   = weight_full(i);
    
    %for n=1:N
    parfor n=1:N      %parallel computing function 
        %-- Generating values for the model
        tau = vTau(n,1);
        T1  = vT1(n,1);
        T2  = T-T1;
        m_bar = root_c_bar*mM_br(n,:)/sqrt_T;
        delta = root_c_tilde*sqrt(tau*(1-tau))*mDelta(n,:)/sqrt_T;
        m1    = m_bar + (1/tau)*delta;
        m2    = m_bar - (1/(1-tau))*delta;
        
        %-- DGP under H0
        X    = squeeze(mX(:,1:k,n));
        bm0  = cumsum(X)/sqrt_T;
        bb0  = bm0-s*bm0(T,:);   % B(s) - s*B(1) brownian bridge
        A0   = bb0(Ttau_l:Ttau_u,:);
        BS_0 = sum(A0.*A0,2)./s_tau_x_1mtau;
        chi0 = bm0(T,:)*bm0(T,:)'; % random number from chi-square dist.
        exp_Stilde_opt0(n,i)= 2*log(mean(exp(.5*term1*(BS_0))));
        exp_S_opt0(n,i)     = term2*chi0+ exp_Stilde_opt0(n,i);   %See Lemma 1
               
        %-- DGP under H1
        X_DGP       = [vOnes(1:T1,1)*m1; vOnes(1:T2,1)*m2]+ X;
        F_T         = sum(X_DGP);
        GMM_S(n,i) = (1/T)*F_T*(F_T');
        
        %-- qLL-S test
        DX_DGP= X_DGP-vOnes*F_T/T;
        vMv   = sum(sum(DX_DGP.^2),2);
        ini   = [X_DGP(1,:);diff(X_DGP)];
        mw    = filter(vr_1,1,ini);%
        %mw    = Mhelp*ini;
        e     = M_vr*mw;
        e2    = sum(sum(e.^2,1),2);
        
        qLL_Stilde(n,i)= -e2*rtilde + vMv;
        qLL_S(n,i)     =  aux1*GMM_S(n,i) + qLL_Stilde(n,i);
        
        bm   = cumsum(X_DGP)/sqrt_T;
        bb   = bm-s*bm(T,:);   % B(s) - s*B(1) brownian bridge
    
        A    = bb(Ttau_l:Ttau_u,:);
        BS_T = sum(A.*A,2)./s_tau_x_1mtau;
        chi  = bm(T,:)*bm(T,:)'; % random number from chi-square dist.
                
        exp_Stilde_opt(n,i)= 2*log(mean(exp(.5*term1*(BS_T))));
        exp_S_opt(n,i) = term2*chi+ exp_Stilde_opt(n,i);   %See Lemma 1
        
        ave_Stilde(n,i) = mean(BS_T);
        exp_Stilde(n,i) = 2*log(mean(exp(.5*BS_T)));
        
        ave_S(n,i) = ave_Stilde(n,i)+chi;
        exp_S(n,i) = exp_Stilde(n,i)+chi;
        
    end
end

%% Computing the critical values
cv_S        = quantile(GMM_S(:,1),1-alpha);
cv_ave_S    = quantile(ave_S(:,1),1-alpha);
cv_exp_S    = quantile(exp_S(:,1),1-alpha);
cv_qLL_S    = quantile(qLL_S(:,1),1-alpha);
cv_exp_S_opt= quantile(exp_S_opt0,1-alpha);

cv_ave_Stilde     = quantile(ave_Stilde(:,1),1-alpha);
cv_exp_Stilde     = quantile(exp_Stilde(:,1),1-alpha); 
cv_qLL_Stilde     = quantile(qLL_Stilde(:,1),1-alpha);  
cv_exp_Stilde_opt = quantile(exp_Stilde_opt0,1-alpha);


% Power Curves
pw_GMM_S = mean(GMM_S>cv_S); 
pw_ave_S = mean(ave_S>cv_ave_S);
pw_exp_S = mean(exp_S>cv_exp_S);
pw_qLL_S = mean(qLL_S>cv_qLL_S);
pw_exp_S_opt = mean(exp_S_opt>ones(size(exp_S_opt,1),1)*cv_exp_S_opt);

pw_ave_Stilde = mean(ave_Stilde>cv_ave_Stilde);
pw_exp_Stilde = mean(exp_Stilde>cv_exp_Stilde);
pw_qLL_Stilde = mean(qLL_Stilde>cv_qLL_Stilde);
pw_exp_Stilde_opt = mean(exp_Stilde_opt>...
        ones(size(exp_Stilde_opt,1),1)*cv_exp_Stilde_opt);
%% - Plot the Power Curves
bbb = min(root_c):.01:max(root_c);
ff  = 0.05;

save  PO_power_b1_k1_omega00   root_c pw* bbb ff 

matlabpool close % parallel computing function 

bbb = min(root_c):.2:max(root_c);
ff  = 0.05;

figure (3)
plot1=plot(root_c,pw_GMM_S(:),...
     root_c,pw_ave_S(:),...
     root_c,pw_exp_S(:),...
     root_c,pw_qLL_S(:),...
     root_c,pw_exp_S_opt(:),...
     'LineWidth',3.0,'MarkerSize',5);
set(plot1(1),'LineWidth',2.5,'Color',[0 0 1]);
set(plot1(2),'LineStyle',':','LineWidth',4,'Color',[0 0 0]);
set(plot1(3),'LineWidth',2.5,'LineStyle','--','Color',[1 0 0]);
set(plot1(4),'LineStyle','-.','LineWidth',2.5,'Color',[0.749 0 0.749]);
set(plot1(5),'LineStyle','-','LineWidth',1,'Color',[0 0 0]);
set(gca,'XTick',[0 5 10 15 20 25],'YTick',[0.2 .40 .60 .80 1.0],'YMinorTick','on','FontSize',14)
axis([min(root_c) max(root_c) 0 1]);
xlabel('$\sqrt{c}$','Interpreter','latex','FontSize',17);
ylabel('rejection probability','FontSize',16);
% legend('S','ave-S','exp-S','qLL-S','opt-S','Location',[0.58 0.20 .42 .30]);
% legend ('boxoff')
saveas(gcf,'b1_k1_omega00', 'pdf')

toc


