% Goal: Plot the power curve of the tests proposed in Sophocles note entitle:
% "Linear IV regression model with time varying first stage
% Linear IV model with 1 endogenous variable. 
% 
% m is the number of endogenous (m=1)
% T is the number of observations (T=2000)
% k is the number of excluded exogenous variables
% We do have parameter instability
% Pi_z_t = =-1 if t<= T/2
% Pi_z_t = =+1 if t> T/2
% The model is: Y   = X*theta + U
%               X   = Z*Pi_z_t   +  V
% or          : Y   = Z*Pi_z_t*theta + E
%               X   = Z*Pi_z_t   +  V
% Y, X, Z are always observed.
% Z is Txkz; theta is scalar; Pi_z is kx1
% vec(U V)  ~ N(0, Sigma kro I(T))
% vec(E V)  ~ N(0, Omega kro I(T))
% Omega is (m+1)x(m+1), covariance matrix, om11=om22=1; om12=om21=rho


tic

clear

%--- Change the path according to your computer
addpath('.../tables','-end')

%% Parameters and Instruments

m   = 1;      % # of endogenous varuables
kz  = 2;      % # of excluded instruments
k   = 2*kz;   % # of potential instruments
T   = 2000;   % # of observations
tau = 0.5;    % break on middle of the sample 
N   = 20000;  % # of Monte Carlo simulations

lambda  = 5;    % concentration parameter
lambda_F = 5.0;
lambda_S = lambda-lambda_F;
%--- Residual Linear IV model
stream_V = RandStream('mt19937ar','Seed',0); %same residuals
mV1      = randn(stream_V,T,N);
mV2      = randn(stream_V,T,N);
%---   Values: structural parameters
theta0 = 0;                % theta
rho    = 0.20;             % corr(v1,v2) ~ degree of endogeneity
Omega  = [1  rho; rho 1];

   
stream_Z = RandStream('mt19937ar','Seed',1); % fix rand generating number
ones_T   = ones(T,1);       % matrix of included instr.
Qzz      = ones_T'*ones_T;

%---Defining Instrument Matrix: only first instrument is relevant
T1     = round(tau*T);
z_tau  = blkdiag(ones_T(1:T1,:),ones_T(T1+1:T,:));
Mz_tau = eye(T)-z_tau*((z_tau'*z_tau)\z_tau');
clear z_tau
Z       = [ones_T Mz_tau*randn(stream_Z,T,kz-1)];
ihZZ    = (Z'*Z)^(-.5);
sqrtmZZ = sqrtm((Z'*Z)/T);
hiZZ_Z  = ihZZ*Z';

Z_tau               = blkdiag(Z(1:T1,:),Z(T1+1:T,:)); % see equation (30)
hiZZ_tau_Z_tau      = ((Z_tau'*Z_tau)^(-.5))*Z_tau';

%Critical values for the loop
alpha  = 0.05;       % significance level

rlamb_theta  = -6:.25:6;
Theta        = (rlamb_theta/sqrt(lambda))';       
st           = size(Theta,1);

%components of the qLL-GMM_S test
c_10    = 10;
term_c  = c_10/(1+c_10); 
rtilde  = 1 - c_10/T;
vr      = rtilde.^(1:1:T)';
vr_1    = [1; vr(1:T-1)];
M_vr    = eye(T) - vr*((vr'*vr)\vr');

%---Storage Matrices

GMM_S        = zeros(N,st); oracle_S     = zeros(N,st); split_S      = zeros(N,st);
GMM_KLM      = zeros(N,st); oracle_KLM   = zeros(N,st); split_KLM    = zeros(N,st);
GMM_JKLM     = zeros(N,st); oracle_JKLM  = zeros(N,st); split_JKLM   = zeros(N,st);
GMM_CLR      = zeros(N,st); oracle_CLR   = zeros(N,st); split_CLR    = zeros(N,st);
GMM_rank     = zeros(N,st); oracle_rank  = zeros(N,st); split_rank   = zeros(N,st);

ave_S        = zeros(N,st); ave_Stilde   = zeros(N,st);
exp_S        = zeros(N,st); exp_Stilde   = zeros(N,st);
qLL_S        = zeros(N,st); qLL_Stilde   = zeros(N,st);

tau_tilde    = zeros(N,st);    % break date estimate

p  = 0.15;                     % proportion of observations discarded pi in [prop,1-prop]tl = ceil(p*T);
tl = ceil(p*T);
tu = round((1-p)*T);
d        = (T*tau<=1:1:T)';
vOnes_kz = ones(1,kz);
vOnes_T  = ones(T,1);

%--- Setting the changes in the means of the process
delta = sqrt(lambda_S/(Qzz*tau*(1-tau)));    % delta = (C_21-C_11) 
C_11  = sqrt(lambda_F/Qzz)-(1-tau)*delta;
C     = C_11+d*delta;
Z_Pi  = ones_T.*C;             % only the first instrument is relevant in the dgp

matlabpool open local 4  %parallel computing function

hiZZs_Zs  = zeros(k,T);
M_HiZZs_Zs  = zeros(k,T,tu-tl+1);
nb_points = (tu-tl+1);
parfor  s = 1:nb_points           %parallel computing function
%for s = 1:nb_points
    helpZ= Z;
    j    = s + tl; 
    Zs   = zeros(T,k);
    Zs(1:j,1:kz)     = helpZ(1:j,:);
    Zs(j+1:T,kz+1:k) = helpZ(j+1:T,:);
    hiZZs_Zs         = ((Zs'*Zs)^(-.5))*Zs';
    M_HiZZs_Zs(:,:,s)= hiZZs_Zs; 
end
%--- Auxiliary Parameters (see page 27)
chol_Omega = chol(Omega);
b0     = [1;-theta0];      
A0     = [theta0; 1];      
b0Omb0 = b0'*Omega*b0;
iOmA0_div_hA0iOmA0 = (Omega\A0)/(sqrt(A0'*(Omega\A0)));

x  = zeros(T,1);
y  = zeros(T,1);

for i=1:st
    theta = Theta(i,1);
    %for n=1:N
    parfor n=1:N           %parallel computing function
        %Generating values for the model
        ev  = [mV2(:,n) mV1(:,n)]*chol_Omega;
        v2  = ev(:,m+1);
        v1  = ev(:,1);
		%--- Linear IV model,  see equation (30) 
        x  = Z_Pi       +  v2;
        y  = Z_Pi*theta +  v1;

        Y  = [y x];
        X1 = Y*b0/(sqrt(b0Omb0));
        X2 = Y*iOmA0_div_hA0iOmA0;

        %-- Weak instruments tests
        F  = hiZZ_Z*X1;
        D  = hiZZ_Z*X2;
        FF = F'*F;
        DD = D'*D;
        FD = F'*D;

        GMM_S(n,i)    = FF;
        GMM_KLM(n,i)  = FD^2/DD;
        GMM_JKLM(n,i) = GMM_S(n,i)-GMM_KLM(n,i);
        GMM_CLR(n,i)  = (1/2)*(FF - DD +...
                        sqrt((FF-DD)^2 + 4*FD^2));
        GMM_rank(n,i) = DD;        
        
        %-- Infeasible weak instruments tests
        oracle_F  = hiZZ_tau_Z_tau*X1;
        oracle_D  = hiZZ_tau_Z_tau*X2;
        oracle_FF = oracle_F'*oracle_F;
        oracle_DD = oracle_D'*oracle_D;
        oracle_FD = oracle_F'*oracle_D;

        oracle_S(n,i)   = oracle_FF;
        oracle_KLM(n,i) = oracle_FD^2/oracle_DD;
        oracle_JKLM(n,i)= oracle_S(n,i)-oracle_KLM(n,i);
        oracle_CLR(n,i) = (1/2)*(oracle_FF - oracle_DD +...
                         sqrt((oracle_FF  - oracle_DD)^2 + 4*oracle_FD^2));
        oracle_rank(n,i)=oracle_DD;
        
        seq_bar_DsDs = zeros(1, nb_points);
        seq_bar_FsDs = zeros(1, nb_points);
        seq_bar_FsFs = zeros(1, nb_points);
        seq_KLMs     = zeros(1, nb_points);
        seq_JKLMs    = zeros(1, nb_points);
        seq_Ss       = zeros(1, nb_points);
        seq_Stildes  = zeros(1, nb_points);
        HiZZs_Zs     = M_HiZZs_Zs;
        for s = 1:nb_points
            hiZZs_Zs = HiZZs_Zs(:,:,s);
            bar_Fs   = hiZZs_Zs*X1;
            bar_Ds   = hiZZs_Zs*X2;
            bar_FsFs = bar_Fs'*bar_Fs;
            bar_DsDs = bar_Ds'*bar_Ds;
            bar_FsDs = bar_Fs'*bar_Ds;

            seq_bar_DsDs(1,s) = bar_DsDs;
            seq_bar_FsDs(1,s) = bar_FsDs;
            seq_bar_FsFs(1,s) = bar_FsFs;
			
			seq_Ss(1,s)      = bar_FsFs;
            seq_Stildes(1,s) = bar_FsFs-FF;
            seq_KLMs(1,s)    = bar_FsDs^2/bar_DsDs;
            seq_JKLMs(1,s)   = seq_bar_FsFs(1,s)- seq_KLMs(1,s);
        end

        ave_S(n,i)      = mean(seq_Ss);
        exp_S(n,i)      = log(mean(exp(.5*seq_Ss)));
        ave_Stilde(n,i) = mean(seq_Stildes);
        exp_Stilde(n,i) = log(mean(exp(.5*seq_Stildes)));
        
        [rank_tau, rt] = max(seq_bar_DsDs);
        bar_FD_htau    = seq_bar_FsDs(rt);
        split_S(n,i)   = seq_bar_FsFs(rt);
        split_KLM(n,i) = seq_KLMs(rt);
        split_JKLM(n,i)= seq_JKLMs(rt);
        split_CLR(n,i) = (1/2)*(split_S(n,i) - rank_tau +...
                          sqrt((split_S(n,i) - rank_tau)^2 + 4*bar_FD_htau^2));
        split_rank(n,i)= rank_tau;       

        tau_tilde(n,i) = rt+tl-1; % estimate break date

        %-- qLL-GMM_S test - see appendix pages 14-15
		X_DGP = (Z.*(X1*vOnes_kz))/sqrtmZZ;    % see equation (S-12)
        DX_DGP= X_DGP-vOnes_T*mean(X_DGP);
        vMv   = sum(sum(DX_DGP.^2),2);         % TSSR_N
        ini   = [X_DGP(1,:);diff(X_DGP)];
        mw    = filter(vr_1,1,ini);
        e     = M_vr*mw;
        e2    = sum(sum(e.^2,1),2);            % TSSR_G
        
        qLL_Stilde(n,i)= -e2*rtilde + vMv; 
        qLL_S(n,i)     =  term_c*GMM_S(n,i) + qLL_Stilde(n,i);
    end
end

tc           = round(st/2);

%%  Critical values for the tests (obtained from the simulation)
cv_S          = quantile(GMM_S(:,tc),1-alpha);
cv_KLM        = quantile(GMM_KLM(:,tc),1-alpha);
cv_JKLM       = quantile(GMM_JKLM(:,tc),1-alpha);
cv_CLR        = zeros(N,st);
cv_oracle_S   = quantile(oracle_S(:,tc),1-alpha);
cv_oracle_KLM = quantile(oracle_KLM(:,tc),1-alpha);
cv_oracle_JKLM= quantile(oracle_JKLM(:,tc),1-alpha);
cv_oracle_CLR = zeros(N,st);
cv_split_S    = quantile(split_S(:,tc),1-alpha);
cv_split_KLM  = quantile(split_KLM(:,tc),1-alpha);
cv_split_JKLM = quantile(split_JKLM(:,tc),1-alpha);
cv_split_CLR  = zeros(N,st);
cv_ave_S      = quantile(ave_S(:,tc),1-alpha);
cv_exp_S      = quantile(exp_S(:,tc),1-alpha);
cv_qLL_S      = quantile(qLL_S(:,tc),1-alpha);
cv_ave_Stilde = quantile(ave_Stilde(:,tc),1-alpha);
cv_exp_Stilde = quantile(exp_Stilde(:,tc),1-alpha); 
cv_qLL_Stilde = quantile(qLL_Stilde(:,tc),1-alpha); 

%---critical values CLR tests
tabCLR  = importdata('table_CLR.mat');
tab2    = squeeze(tabCLR(1,2,:,[1 191])); %Critical Values GMM_CLR-test
tab4    = squeeze(tabCLR(1,4,:,[1 191])); %Critical Values Oracle GMM_CLR-test
ones_st = ones(1,st);
si      = 2;
sl      = size(tab2,1);
ones_tab= ones(sl-si+1,1);
loc     = tab2(si:sl,1)*ones_st;
loc_tau = tab4(si:sl,1)*ones_st;

parfor i=1:N                            %parallel computing function
    Tab2 = tab2;
    Tab4 = tab4;
    ss     = sum(loc     < ones_tab*GMM_rank(i,:))+1;
    ss_tau = sum(loc_tau < ones_tab*oracle_rank(i,:))+1;
    sst    = sum(loc_tau < ones_tab*split_rank(i,:))+1;
    
	cv_CLR(i,:)       =Tab2(ss,2);
    cv_oracle_CLR(i,:)=Tab4(ss_tau,2);
    cv_split_CLR(i,:) =Tab4(sst,2);
end


pw_S          = mean(GMM_S>cv_S);
pw_KLM        = mean(GMM_KLM>cv_KLM);
pw_JKLM       = mean(GMM_JKLM>cv_JKLM);
pw_CLR        = mean(GMM_CLR>cv_CLR);
pw_oracle_S   = mean(oracle_S>cv_oracle_S);
pw_oracle_KLM = mean(oracle_KLM>cv_oracle_KLM);
pw_oracle_JKLM= mean(oracle_JKLM>cv_oracle_JKLM);
pw_oracle_CLR = mean(oracle_CLR>cv_oracle_CLR);
pw_split_S    = mean(split_S>cv_split_S);
pw_split_KLM  = mean(split_KLM>cv_split_KLM);
pw_split_JKLM = mean(split_JKLM>cv_split_JKLM);
pw_split_CLR  = mean(split_CLR>cv_split_CLR);
pw_ave_S      = mean(ave_S>cv_ave_S);
pw_exp_S      = mean(exp_S>cv_exp_S);
pw_qLL_S      = mean(qLL_S>cv_qLL_S);
pw_ave_Stilde = mean(ave_Stilde>cv_ave_Stilde);
pw_exp_Stilde = mean(exp_Stilde>cv_exp_Stilde);
pw_qLL_Stilde = mean(qLL_Stilde>cv_qLL_Stilde);

mt_tilde=mean(tau_tilde);

%% - Plot the Power Curves
bbb = -6:.1:6;
ff  = 0.05;

save  Power_k2_rho20_lambda_s_00  Theta pw* rlamb_theta ...
      mt_tilde Theta bbb ff  T

matlabpool close            %parallel computing function
  
%--- Change the path according to your computer
rmpath('.../tables')

bbb = min(rlamb_theta):.2:max(rlamb_theta);
ff  = 0.05;

figure (10+1)
plot1=plot(...
     rlamb_theta,pw_ave_S(:),...
     rlamb_theta,pw_exp_S(:),...
     rlamb_theta,pw_qLL_S(:),...
     rlamb_theta,pw_oracle_CLR(:),...
     rlamb_theta,pw_split_CLR(:),...
     'LineWidth',3.0,'MarkerSize',5);
set(plot1(1),'LineStyle',':','LineWidth',4,'Color',[0 0 0]);
set(plot1(2),'LineWidth',3.0,'LineStyle','--','Color',[1 0 0]);
set(plot1(3),'LineStyle','-.','LineWidth',3.0,'Color',[0.749 0 0.749]);
set(plot1(4),'LineStyle','-','LineWidth',1,'Color',[0 0 0]);
set(plot1(5),'LineStyle','--','LineWidth',1,'Color',[0 0 0]);
set(gca,'XTick',[-6 -4 -2 0 2 4 6 ],'YTick',[0.2 .40 .60 .80 1.0],'FontSize',14)
axis([min(rlamb_theta) max(rlamb_theta) 0 1]);
xlabel('$\sqrt{\lambda}(\theta-\theta_0)$','Interpreter','latex','FontSize',16);
ylabel('rejection probability','FontSize',16);
saveas(gcf,'k2_rho20_lambda_s_00', 'pdf')

toc

