% Model: New Keynesian Phillips Curve
% Parameters are calibrate to minic US data
% Goal: Compare size among GMM_S, gen-S, gen-Stilde and split sample tests 
% 
% Data Degenating Process:
% pi_t = beta*E[pi_{t+1}|t] + gamma*x_t + e_t
% x_t  = rho_{1t}*x_{t-1}   + rho_{2t}*x_{t-2} + v_t
% 
% Identification needs gamma~=0 and r_{2t}~=0;
%
% Solving the system we derived 
% pi_t = al_t*x_t  + al_t*r_{2,t}*beta*x_{t-1} + e_t
% x_t  = rho_{1,t}*x_{t-1} + rho_{2,t}*x_{t-2} + v_t
% 
% rho_{1,t}= rho*(1-rho_{2,t})
% rho_{2,t}=  0.063- kappa*0.075  if t<=T/2;
%          =  0.063+ kappa*0.075  if  t>T/2;
%
% s_ee  = .30 + phi*0.034; if t<=T/2        % var(epsilon) before break
% s_ee_2= .30 - phi*0.034; if t>T/2         % var(epsilon) after break
% s_vv  = .01;                    % var(v)                     
% s_ev  = -0.012;                 % cov(epsilon,v)
%
% Parameters of the DGP
% beta=1; gamma = 1/6; rho = 0.9
% al_t = gamma/(1 - beta*(rho_{1t} - beta* rho_{2t}));
%
% GMM approach:
% moments: h_t(b,lam) = pi_t - b*pi_{t+1} - lam* x_t;
%                     = e_t - b* nu_{t+1}, nu_t= e_t + v_t*al_t
%          f_t(b,lam) = Z_t'*h_t(b,lam);
% Z_t = (x_{t-1}, x_{t-2}) - Instruments

tic
clear
addpath('.../functions','-end')
addpath('.../tables','-end')

%% Parameters
g     = 2;               % # of endogenous regressor
p     = 2;               % # of lags in the autoregressive model
gp    = g*p;
l     = 1;               % pi_{t+1}
ini   = 5;               % # of initial observations 
T     = 180;             % # of available observations
TLL   = T + ini ;        % # of generated observations
Tbreak= round(T/2) + ini;% position of the break
N     = 20000;            % # of  simulations

stream_V = RandStream('mt19937ar','Seed',0); %same residuals
mV1      = randn(stream_V,TLL,N);
mV2      = randn(stream_V,TLL,N);
 
%--- Structural parameters
beta0 = 1;                                 % beta
alpha0= 2/3;
gamma0= ((1-alpha0)*(1-alpha0*beta0))/(alpha0);           % lambda 
b     = [1;-beta0;-gamma0];

kappa = 0.00;              % magnitude of the break of rho_2
phi   = 0.00;              % magnitude of the break of sigma_ee

rho   = .9;                       % see line 16
%--- Reduced Form parameters
rho_2 = [0.063- kappa*0.075;
         0.063+ kappa*0.075]; % see lines 16-18
rho_1 = rho*(1-rho_2);

den   = (1- beta0*(rho_1 + beta0*rho_2));
a_0   =  gamma0./den;
eta_1 = (gamma0*rho_1+ (beta0*gamma0*rho_2))./den;
eta_2 = (gamma0./den).*rho_2;

sigma_ee=[.30 + phi*0.034;            % var(epsilon) t<=T/2
          .30 - phi*0.034];           % var(epsilon) t> T/2
Sigma1  =[sigma_ee(1)  -0.012;
          -0.012         0.01];
CSigma1 = chol(Sigma1);  % Var-Cov  t<=T/2
CSigma2 = chol([sigma_ee(2)    -0.012;  % Var-Cov  t> T/2
                 -0.012          0.01]);    
%--- Computing the variance of the stationary process:
%--- Y_t= [pi_t x_t pi_{t-1} x_{t-1}]
% Variance of AR(1) representation
Q       = blkdiag(Sigma1,zeros(g*(p-1)));  
% Companion Matrix see equation (10.1.10) Hamilton (1994)
F       = [0 eta_1(1)     0 eta_2(1);     
           0 rho_1(1)     0 rho_2(1);
           eye(g*(p-1)) zeros(g*(p-1),g)];
% vec of variance of Y_t see equation (10.2.18) Hamilton (1994)       
vecPsi  = (eye(gp^2)-kron(F,F))\Q(:);            
Psi     =  reshape(vecPsi,gp,gp);  
cholPsip =  chol(Psi)';

str0  = RandStream('swb2712','Seed',1);    % initial values for x
xT    = zeros(TLL,1);                      % labor share
piT   = zeros(TLL,1);                      % inflation

k    = 2;           % # of moment conditions
m    = 1;           % # of endogenous regressors 
kk   = 2*k; 
ky   = size(b,1);   %size(Y,2);
kky  = k*ky;

%---obs. left out for the  struct. break tests
%--- if pr=.15, then pp=1; if pr=.2, then pp=2;
 
pr      = 0.15;       
tl      = ceil(pr*T);
tu      = round((1-pr)*T);

I_k   = eye(k); I_m   = eye(m); I_kk  = eye(kk);
I_T   = eye(T); I_2 = eye(2);

ons_k   = ones(1,k); ons_T = ones(T,1);
Mx      = I_T-ons_T*((ons_T'*ons_T)\ons_T');

%-- Variables  of the qLL-S test
c10     = 10; term_c10= c10/(1+c10);
b_bar = kron(b,I_k);
db    = [0;
         0;
         (1-alpha0^2)/(alpha0^2)];

%--- Permutation Matrices (computing Var(vec(s.ZY)), see equation (S-16) 
% Per.C = permutation columns; 
% Per.R = kron(E',I.k.z);  %permutation rows
[Per.C,Per.R]= permut_matrix(ky,2*ky,I_k);

%---Zeros Matrices
D_hat = zeros(k,m)      ; s.D_hat = zeros(kk,m); 
s.S   = zeros(1,tu-tl+1); s.Stilde= zeros(1,tu-tl+1); 
s.KLM = zeros(1,tu-tl+1); s.DVD   = zeros(1,tu-tl+1);
s.D   = zeros(kk,m,tu-tl+1);
s.V_D = zeros(tu-tl+1,kk*m,kk*m);
                      
%--Weak Instruments Tests
GMM = struct('S',zeros(N,1),'KLM',zeros(N,1),...
            'JKLM',zeros(N,1),'CLR',zeros(N,1),'rk',zeros(N,1));
%--Split Sample Tests
Split = struct('S',zeros(N,1), 'KLM',zeros(N,1),...
              'JKLM',zeros(N,1),'CLR',zeros(N,1),'rk',zeros(N,1));
%--Generalized Tests
GenS = struct('ave', zeros(N,1),'exp',zeros(N,1),'qLL',zeros(N,1));
%--Stability Tests
GenStilde= struct('ave', zeros(N,1),'exp',zeros(N,1),'qLL',zeros(N,1));  
  
for n=1:N
    %--- DGP Residuals
    
    V  = [ [mV2(1:Tbreak,n)     mV1(1:Tbreak,n)    ]*CSigma1;
           [mV2(Tbreak+1:end,n) mV1(Tbreak+1:end,n)]*CSigma2 ];
    e  = V(:,1);
    v  = V(:,2);
    %--- Sampling from the Stationary Distribution
    Y_0    = cholPsip*randn(str0,g*p,1);
    xT(1,1)= Y_0(gp,1);
    xT(2,1)= Y_0(g,1);
    %--- DGP
    for t=(p+1):Tbreak
        xT(t,1) = rho_1(1)*xT(t-1,1)+ rho_2(1)*xT(t-2,1)+ v(t,1);
        piT(t,1)= eta_1(1)*xT(t-1,1)+ eta_2(1)*xT(t-2,1)+ e(t,1)+a_0(1)*v(t,1);
    end
    
    for t=(Tbreak+1):TLL
        xT(t,1) = rho_1(2)*xT(t-1,1)+ rho_2(2)*xT(t-2,1)+ v(t,1);
        piT(t,1)= eta_1(2)*xT(t-1,1)+ eta_2(2)*xT(t-2,1)+ e(t,1)+a_0(2)*v(t,1);
    end
    %--- Instrument Set
    Z   = [xT(ini-l:TLL-2,1) xT(ini-2:TLL-3,1)];
    Zp  = Z';
    
    Y    = [piT(ini:TLL-1,1)-piT(ini-1:TLL-2,1) piT(ini+l:TLL,1)-piT(ini-1:TLL-2,1)...
         	xT(ini:TLL-1,1)];
    
	Sigma_hat = HAC_Sigma_hat(Mx*Y,Z,ky,k,T);
    V_ff  = b_bar'*Sigma_hat*b_bar;% Variance S and qLL-S tests see equation (S-7)
    hV_ff = sqrtm(V_ff);           % Symmetric square root
    
    Yb   = Y*b;  
    Ydb  = Y*db; 
    H    = Z*(V_ff\Zp);
    e_hat= Yb;          % see equation (S-11)
    %--- Normalized moment condition
    F_hat             =(Z.*(e_hat*ons_k))/hV_ff; % see equation (S-12) 
    GenStilde.qLL(n,1)= qLL_S_tilde(F_hat,c10);
    
    [U, ~] = svd(I_k);
    L      = U(1:k,1:k);  % L*L' = I_k & L'*L = I_k
    Xi_hat = L'*(hV_ff\Zp*Yb);
    Xi_hatp= Xi_hat';          % see statistic S_T on page 13    
    
    GMM.S(n,1)   = (1/T)*(Xi_hatp*Xi_hat);
    GenS.qLL(n,1)= term_c10*GMM.S(n,1) + GenStilde.qLL(n,1);
    
            %---Jacobian Matrix for KLM test
    L_tilde = hV_ff\L;

    Aux1  = kron(b,L_tilde); 
    Aux2  = kron(db,L_tilde);     % == db_bar*kron(I.p,L_tilde);
    V_xx  = I_k;              % == Aux1'*Sigma_hat*Aux1;
    V_qx  = Aux2'*Sigma_hat*Aux1;
    V_qq  = Aux2'*Sigma_hat*Aux2; 
    V_qqx = V_qq - V_qx*(V_xx\V_qx');
           
    q_hat = L_tilde'*Zp*Ydb; h = 0;
    
    for r=1:m
        D_hat(:,h+1:r)= q_hat(:,r)...
            - V_qx( k*h+1: k*r,:)*(V_xx\(Xi_hat));
        h=h+1;
    end
    D_hat = D_hat/T; 
    D_hatp= D_hat';
    
    GMM.KLM(n,1)= (1/T)*Xi_hatp*D_hat*((D_hatp*D_hat)\(D_hatp*(Xi_hat)));
    GMM.JKLM(n,1)= GMM.S(n,1)-GMM.KLM(n,1);
            
    %--- rank statistic for CLR test:
    rk          = ranktest_svd(D_hat,I_m, V_xx^(-.5),V_qqx,m-1,T);
    GMM.rk(n,1) = rk;
    GMM.CLR(n,1)= .5*(GMM.S(n,1)-rk +...
                  sqrt((GMM.S(n,1)+rk)^2 - 4*GMM.JKLM(n,1)*rk));
    
    %--- Loop for computing Split-sample S && estimation of break date      
    s.b_bar = kron(b,I_kk);
    s.b_barp= s.b_bar';
    
    for t_break = tl:tu
        ts   = t_break-tl+1;
        s.Z  = blkdiag(Z(1:t_break,:),Z(t_break+1:T,:)); % see equation (S-13)
        s.Zp = s.Z';
        st   = t_break/T;
        
        %--- DANGER: s.Sigma_hat != var(s.Zp*Y)
        s.Sigma_hat = blkdiag(st*Sigma_hat,(1-st)*Sigma_hat);
        s.Sigma_hat=  Per.R*s.Sigma_hat*Per.C;
        %---- Var( T^-.5 Zbar*Yb )
        s.V_ff=  s.b_barp*s.Sigma_hat*s.b_bar; % see equation (S-16)
        
        s.hV_ff= sqrtm(s.V_ff);
        %--- SVD of the orthogonal projection matrix
        [s.U, ~]= svd(I_kk);
        s.L     = s.U(1:kk,1:kk); % s.L*s.L' = I_kk
                                  % s.L'*s.L = I(2*k)
        s.X_T    = s.hV_ff\s.Zp*Yb;
        s.Xi_hat = s.L'*s.X_T;
        s.Xi_hatp= s.Xi_hat';
        
        s.S(ts)      = (1/T)*s.Xi_hatp*(s.Xi_hat);
        s.Stilde(ts) = s.S(ts)-GMM.S(n,1);
        
        s.L_tilde = s.hV_ff\s.L;
        s.Aux1    = kron( b,s.L_tilde);
        s.Aux2    = kron(db,s.L_tilde);
        
        s.V_xx = I_kk;               % == s.Aux1'*s.Sigma_hat*s.Aux1;
        s.V_qx = s.Aux2'*s.Sigma_hat*s.Aux1;
        s.V_qq = s.Aux2'*s.Sigma_hat*s.Aux2;
        s.V_qqx= s.V_qq - s.V_qx*(s.V_xx\s.V_qx');
        
        s.q_hat= s.L_tilde'*s.Zp*Ydb; h = 0;
        
        for r=1:m
            s.D_hat(:,h+1:r)= s.q_hat(:,r)-...
                s.V_qx((kk)*h+1:(kk)*r,:)*(s.V_xx\s.Xi_hat);
            h=h+1;
        end
        s.D_hat  = s.D_hat/T;
        s.D_hatp = s.D_hat';
        
        s.KLM(ts)=(1/T)*...
            s.Xi_hatp*s.D_hat*((s.D_hatp*s.D_hat)\(s.D_hatp*s.Xi_hat));
        s.DVD(ts)= s.D_hat(:)'*(s.V_qqx\s.D_hat(:));
        
        % Saved s.D_hat and s.V_qqx to be used in computing split CLR
        s.D(:,ts)  = s.D_hat;
        s.V_D(ts,:,:)= s.V_qqx;
        
        
    end
    GenStilde.ave(n,1) = mean(s.Stilde);
    GenStilde.exp(n,1) = log(mean(exp(.5*s.Stilde)));
    
    GenS.ave(n,1) = mean(s.S);
    GenS.exp(n,1) = log(mean(exp(.5*s.S)));
    %---Estimating the break date
    [~, ir]       = sort(s.DVD,'descend');
    
    Split.S(n,1)   = s.S(ir(1));
    Split.KLM(n,1) = s.KLM(ir(1));     
    Split.JKLM(n,1)= Split.S(n,1)-Split.KLM(n,1);
    
    sD_hat = squeeze(s.D(:,ir(1)));
    sV_qqx = squeeze(s.V_D(ir(1),:,:));
            
    rkt =  ranktest_svd(sD_hat,I_m, s.V_xx^(-.5),sV_qqx,m-1,T);
    Split.rk(n,1) = rkt;
    Split.CLR(n,1)= .5*(Split.S(n,1)-rkt+ ...
                sqrt((Split.S(n,1) + rkt)^2 - 4*Split.JKLM(n,1)*rkt));   
end

%---Critical values for the full sample and stability tests
alpha    = [0.10 0.05];   % significance level
sa       = size(alpha,2);

[CV_GenS, CV_GenStilde]=Cvalue_GenS(k,k,alpha);

CV_GMM.S       = chi2inv(1-alpha,k);
CV_Split.S     = chi2inv(1-alpha,2*k);
CV_Split.KLM   = chi2inv(1-alpha,m);
CV_Split.JKLM  = chi2inv(1-alpha,2*k-m);
CV_Split.KJLMK = chi2inv(1-.8*alpha,m);
CV_Split.KJLMJ = chi2inv(1-.2*alpha,2*k-m);


%% --- Saving the results
ones_sa   = ones(1,sa);
ones_N    = ones(N,1);

Size.GenS.ave = 100*(mean(ones_N*CV_GenS.ave<GenS.ave*ones_sa));
Size.GenS.exp = 100*(mean(ones_N*CV_GenS.exp<GenS.exp*ones_sa));
Size.GenS.qLL = 100*(mean(ones_N*CV_GenS.qLL<GenS.qLL*ones_sa));

Size.GenStilde.ave = 100*(mean(ones_N*CV_GenStilde.ave<GenStilde.ave*ones_sa));
Size.GenStilde.exp = 100*(mean(ones_N*CV_GenStilde.exp<GenStilde.exp*ones_sa));
Size.GenStilde.qLL = 100*(mean(ones_N*CV_GenStilde.qLL<GenStilde.qLL*ones_sa));

Size.GMM.S      = 100*(mean(ones_N*CV_GMM.S	<GMM.S*ones_sa));
Size.Split.S    = 100*(mean(ones_N*CV_Split.S<Split.S*ones_sa));
Size.Split.KLM  = 100*(mean(ones_N*CV_Split.KLM <Split.KLM*ones_sa));
Size.Split.JKLM = 100*(mean(ones_N*CV_Split.JKLM <Split.JKLM*ones_sa));

Size.Split.KJLM   = 100*(mean ((ones_N*CV_Split.KJLMK<Split.KLM*ones_sa)...
                           |(ones_N*CV_Split.KJLMJ<Split.JKLM*ones_sa)));

%---Critical values split CLR test
CV.Split.CLR = Cvalue_CLR(m,kk,alpha,Split.rk);

Size.Split.CLR   = 100*(mean(CV.Split.CLR<Split.CLR*ones_sa));

test = ['GMM-S       ';
        'ave-S       ';
        'exp-S       ';
        'qLL-S       ';
        'split-S     ';
        'split-KJLM  ';
        'split-CLR   ' ];
size = [Size.GMM.S;	
        Size.GenS.ave; 
        Size.GenS.exp;
        Size.GenS.qLL;
        Size.Split.S 
        Size.Split.KJLM;
		Size.Split.CLR];

fprintf(1,'\n HAC pre-whitening, lag=prop, %2.00f obs., k=%2.00f\n',T, k)
fprintf(1,' alpha=%2.2f, kappa = %2.2f, phi = %2.2f', alpha0, kappa, phi )
fprintf(1,'\n             Significance Level        \n')
fprintf(1,'tests         %4.0f%%    %4.0f%%  \n', 100*alpha)
for i=1:7
fprintf(1,'%s | %5.2f    %5.2f|  \n',...
    test(i,:) , size(i,:))
end
save  nkpc_kap_000 Size* alpha kappa phi alpha0 test T k

rmpath('.../functions','-end')
rmpath('.../tables','-end')


toc

